package social

import (
	"net/url"
	"regexp"
	"strings"

	"git.nakama.town/fmartingr/butterrobot/internal/model"
	"git.nakama.town/fmartingr/butterrobot/internal/plugin"
)

// InstagramExpander transforms instagram.com links to ddinstagram.com links
type InstagramExpander struct {
	plugin.BasePlugin
}

// New creates a new InstagramExpander instance
func NewInstagramExpander() *InstagramExpander {
	return &InstagramExpander{
		BasePlugin: plugin.BasePlugin{
			ID:             "social.instagram",
			Name:           "Instagram Link Expander",
			Help:           "Automatically converts instagram.com links to alternative domain links and removes tracking parameters. Configure 'domain' option to set replacement domain (default: ddinstagram.com)",
			ConfigRequired: true,
		},
	}
}

// OnMessage handles incoming messages
func (p *InstagramExpander) OnMessage(msg *model.Message, config map[string]interface{}, cache model.CacheInterface) []*model.MessageAction {
	// Skip empty messages
	if strings.TrimSpace(msg.Text) == "" {
		return nil
	}

	// Get replacement domain from config, default to ddinstagram.com
	replacementDomain := "ddinstagram.com"
	if domain, ok := config["domain"].(string); ok && domain != "" {
		replacementDomain = domain
	}

	// Regex to match instagram.com links
	// Match both http://instagram.com and https://instagram.com formats
	// Also match www.instagram.com
	instagramRegex := regexp.MustCompile(`https?://(www\.)?(instagram\.com)/[^\s]+`)

	// Check if the message contains an Instagram link
	if !instagramRegex.MatchString(msg.Text) {
		return nil
	}

	// Replace instagram.com with configured domain in the message and clean query parameters
	transformed := instagramRegex.ReplaceAllStringFunc(msg.Text, func(link string) string {
		// Parse the URL
		parsedURL, err := url.Parse(link)
		if err != nil {
			// If parsing fails, just do the simple replacement
			return link
		}

		// Ensure we don't change links that already come from the replacement domain
		if parsedURL.Host != "instagram.com" && parsedURL.Host != "www.instagram.com" {
			return link
		}

		// Change the host to the configured domain
		parsedURL.Host = replacementDomain

		// Remove query parameters
		parsedURL.RawQuery = ""

		// Return the cleaned URL
		return parsedURL.String()
	})

	// Create response message
	response := &model.Message{
		Text:    transformed,
		Chat:    msg.Chat,
		ReplyTo: msg.ID,
		Channel: msg.Channel,
	}

	action := &model.MessageAction{
		Type:    model.ActionSendMessage,
		Message: response,
		Chat:    msg.Chat,
		Channel: msg.Channel,
	}

	return []*model.MessageAction{action}
}
